@@ -3,18 +3,18 @@
 #include "VDetector.hh"
 #include "execNEST.hh"
 #include "DetectorExample_XENON10.hh"
-#include "LUX_Run03.hh"
+#include "RandomGen.hh"
 #include <pybind11/numpy.h>
 #include <pybind11/stl_bind.h>
 #include <pybind11/stl.h>
 #include <pybind11/functional.h>
 
-
-
 namespace py = pybind11;
 
 PYBIND11_MODULE(nestpy, m) {
   //	Binding for YieldResult struct
+  RandomGen::rndm()->SetSeed( time(nullptr) );
+
   py::class_<NEST::YieldResult>(m, "YieldResult", py::dynamic_attr())
     .def(py::init<>())
 	.def_readwrite("PhotonYield", &NEST::YieldResult::PhotonYield)
@@ -151,14 +151,14 @@ PYBIND11_MODULE(nestpy, m) {
 	 .def("FitTBA", &DetectorExample_XENON10::FitTBA)
 	 .def("OptTrans", &DetectorExample_XENON10::OptTrans)
 	 .def("SinglePEWaveForm", &DetectorExample_XENON10::SinglePEWaveForm);
-
-  //	Binding for example LUX_Run03
-  py::class_<DetectorExample_LUX_RUN03, VDetector, std::unique_ptr<DetectorExample_LUX_RUN03, py::nodelete>>(m, "LUX_RUN3")
-     .def(py::init<>())
-	 .def("Initialization", &DetectorExample_LUX_RUN03::Initialization)
-	 .def("FitTBA", &DetectorExample_LUX_RUN03::FitTBA)
-	 .def("OptTrans", &DetectorExample_LUX_RUN03::OptTrans)
-	 .def("SinglePEWaveForm", &DetectorExample_LUX_RUN03::SinglePEWaveForm);
+//
+//  //	Binding for example LUX_Run03
+//  py::class_<DetectorExample_LUX_RUN03, VDetector, std::unique_ptr<DetectorExample_LUX_RUN03, py::nodelete>>(m, "LUX_RUN3")
+//     .def(py::init<>())
+//	 .def("Initialization", &DetectorExample_LUX_RUN03::Initialization)
+//	 .def("FitTBA", &DetectorExample_LUX_RUN03::FitTBA)
+//	 .def("OptTrans", &DetectorExample_LUX_RUN03::OptTrans)
+//	 .def("SinglePEWaveForm", &DetectorExample_LUX_RUN03::SinglePEWaveForm);
 
   //	Binding for the NESTcalc class
   py::class_<NEST::NESTcalc, std::unique_ptr<NEST::NESTcalc, py::nodelete>>(m, "NESTcalc")
@@ -172,6 +172,14 @@ PYBIND11_MODULE(nestpy, m) {
 	.def("PhotonTime", &NEST::NESTcalc::PhotonTime)
 	.def("AddPhotonTransportTime", &NEST::NESTcalc::AddPhotonTransportTime)
 	.def("GetPhotonTimes", &NEST::NESTcalc::GetPhotonTimes)
+    .def("GetYieldKr83m",
+	     &NEST::NESTcalc::GetYieldKr83m,
+	     py::arg("energy") = 41.5,
+	     py::arg("density") = 2.9,
+	     py::arg("drift_field") = 124,
+	     py::arg("maxTimeSeparation") = 2000,
+	     py::arg("deltaT_ns") = -999.
+	 )
     .def("GetYields",
 	     &NEST::NESTcalc::GetYields,
 	     py::arg("interaction") = NEST::INTERACTION_TYPE::NR,
